@@ -1,9 +1,8 @@
 use super::COLORS;
-use crate::widget::{Cell, Row, Table, TableState};
+use crate::widget::{Cell, Gauge, Row, Table, TableState};
 use crossterm::event::KeyModifiers;
 use gonk_core::Index;
 use gonk_player::Player;
-use std::time::Duration;
 use tui::layout::{Alignment, Constraint, Direction, Layout, Rect};
 use tui::style::{Color, Modifier, Style};
 use tui::text::{Span, Spans};
@@ -69,56 +68,20 @@ impl Queue {
 }
 
 impl Queue {
-    fn handle_mouse<B: Backend>(&mut self, f: &mut Frame<B>) {
-        //Songs
-        if let Some((_, row)) = self.clicked_pos {
-            let size = f.size();
-            let height = size.height as usize;
-            let len = self.player.songs.len();
-            if height > 7 {
-                if height - 7 < len {
-                    //TODO: I have no idea how to figure out what index i clicked on
-                } else {
-                    let start_row = 5;
-                    if row >= start_row {
-                        let index = (row - start_row) as usize;
-                        if index < len {
-                            self.ui.select(Some(index));
-                        }
-                    }
-                }
-            }
-        }
-
-        //Seeker
-        if let Some((column, row)) = self.clicked_pos {
-            let size = f.size();
-            if size.height - 3 == row
-                || size.height - 2 == row
-                || size.height - 1 == row && column >= 3 && column < size.width - 2
-            {
-                let ratio = f64::from(column - 3) / f64::from(size.width);
-                let duration = self.player.duration;
-                let new_time = duration * ratio;
-                self.player.seek_to(Duration::from_secs_f64(new_time));
-            }
-            self.clicked_pos = None;
-        }
-    }
     pub fn draw<B: Backend>(&mut self, f: &mut Frame<B>) {
         let chunks = Layout::default()
             .direction(Direction::Vertical)
             .constraints([
                 Constraint::Length(3),
-                Constraint::Min(0),
-                Constraint::Length(2),
+                Constraint::Min(10),
+                Constraint::Length(3),
             ])
             .split(f.size());
 
-        self.handle_mouse(f);
         self.draw_header(f, chunks[0]);
-        self.draw_songs(f, chunks[1]);
-        self.draw_seeker(f, chunks[2]);
+        self.draw_body(f, chunks[1]);
+        //TODO: if allow for old and new seeker
+        self.draw_new_seeker(f, chunks[2]);
     }
     fn draw_header<B: Backend>(&mut self, f: &mut Frame<B>, chunk: Rect) {
         //Render the borders first
@@ -132,7 +95,7 @@ impl Queue {
             String::from("╭─Stopped")
         } else if !self.player.is_paused() {
             let duration = self.player.duration;
-            let elapsed = self.player.elapsed().as_secs_f64();
+            let elapsed = self.player.elapsed();
 
             let mins = elapsed / 60.0;
             let rem = elapsed % 60.0;
@@ -162,24 +125,16 @@ impl Queue {
     }
     fn draw_title<B: Backend>(&mut self, f: &mut Frame<B>, chunk: Rect) {
         let center = if let Some(song) = self.player.songs.selected() {
-            //I wish that paragraphs had clipping
-            //I think constraints do
-            //I could render the -| |- on a seperate layer
-            //outside of the constraint which might work better?
-
-            //clip the name so it doesn't overflow
             let mut artist = song.artist.clone();
             let mut name = song.name.clone();
             let width = chunk.width.saturating_sub(45) as usize;
-            let mut i = 0;
 
             while artist.len() + name.len() + "-| - |-".len() > width {
-                if i % 2 == 0 {
+                if artist.len() > name.len() {
                     artist.pop();
                 } else {
                     name.pop();
                 }
-                i += 1;
             }
 
             vec![
@@ -203,11 +158,12 @@ impl Queue {
         let center = Paragraph::new(center).alignment(Alignment::Center);
         f.render_widget(center, chunk);
     }
-    fn draw_songs<B: Backend>(&self, f: &mut Frame<B>, chunk: Rect) {
+    fn draw_body<B: Backend>(&mut self, f: &mut Frame<B>, chunk: Rect) {
+        let border = Borders::LEFT | Borders::BOTTOM | Borders::RIGHT;
         if self.player.songs.is_empty() {
             return f.render_widget(
                 Block::default()
-                    .borders(Borders::LEFT | Borders::RIGHT)
+                    .borders(border)
                     .border_type(BorderType::Rounded),
                 chunk,
             );
@@ -273,19 +229,22 @@ impl Queue {
 
                     //Current selection
                     if ui_index != playing_index {
-                        let song = songs.get(ui_index).unwrap();
-                        let row = Row::new(vec![
-                            Cell::from(""),
-                            Cell::from(song.number.to_string())
-                                .style(Style::default().bg(COLORS.track)),
-                            Cell::from(song.name.clone()).style(Style::default().bg(COLORS.title)),
-                            Cell::from(song.album.clone()).style(Style::default().bg(COLORS.album)),
-                            Cell::from(song.artist.clone())
-                                .style(Style::default().bg(COLORS.artist)),
-                        ])
-                        .style(Style::default().fg(Color::Black));
-                        items.remove(ui_index);
-                        items.insert(ui_index, row);
+                        if let Some(song) = songs.get(ui_index) {
+                            let row = Row::new(vec![
+                                Cell::from(""),
+                                Cell::from(song.number.to_string())
+                                    .style(Style::default().bg(COLORS.track)),
+                                Cell::from(song.name.clone())
+                                    .style(Style::default().bg(COLORS.title)),
+                                Cell::from(song.album.clone())
+                                    .style(Style::default().bg(COLORS.album)),
+                                Cell::from(song.artist.clone())
+                                    .style(Style::default().bg(COLORS.artist)),
+                            ])
+                            .style(Style::default().fg(Color::Black));
+                            items.remove(ui_index);
+                            items.insert(ui_index, row);
+                        }
                     }
                 }
             }
@@ -311,37 +270,90 @@ impl Queue {
             )
             .block(
                 Block::default()
-                    .borders(Borders::LEFT | Borders::RIGHT)
+                    .borders(border)
                     .border_type(BorderType::Rounded),
             )
             .widths(&con);
 
+        // handle mouse
+        if let Some((_, height)) = self.clicked_pos {
+            let (start, _) = t.get_row_bounds(ui_index, t.get_row_height(chunk));
+            //the header is 5 units long
+            if height >= 5 {
+                let index = (height - 5) as usize + start;
+                //don't select out of bounds
+                if index < self.player.songs.len() {
+                    self.ui.select(Some(index));
+                    self.clicked_pos = None;
+                }
+            }
+        }
+
         //required to scroll songs
         let mut state = TableState::new(ui_index);
         f.render_stateful_widget(t, chunk, &mut state);
     }
-    //TODO: try a new seeker design
-    fn draw_seeker<B: Backend>(&self, f: &mut Frame<B>, chunk: Rect) {
+    fn draw_new_seeker<B: Backend>(&mut self, f: &mut Frame<B>, chunk: Rect) {
+        let block = Block::default()
+            .borders(Borders::ALL)
+            .border_type(BorderType::Rounded);
+
+        let elapsed = self.player.elapsed();
+        let duration = self.player.duration;
+
+        let seeker = format!(
+            "{:02}:{:02}/{:02}:{:02}",
+            (elapsed / 60.0).floor(),
+            elapsed.trunc() as u32 % 60,
+            (duration / 60.0).floor(),
+            duration.trunc() as u32 % 60,
+        );
+
+        let ratio = self.player.elapsed() / self.player.duration;
+        let ratio = if ratio.is_nan() {
+            0.0
+        } else {
+            ratio.clamp(0.0, 1.0)
+        };
+
+        let g = Gauge::default()
+            .block(block)
+            .gauge_style(Style::default().fg(Color::White))
+            .ratio(ratio)
+            .label(seeker);
+
+        f.render_widget(g, chunk);
+
+        //mouse
+        if let Some((width, height)) = self.clicked_pos {
+            let size = f.size();
+            if size.height - 3 == height || size.height - 2 == height || size.height - 1 == height {
+                let ratio = f64::from(width) / f64::from(size.width);
+                let duration = self.player.duration;
+                let new_time = duration * ratio;
+                self.player.seek_to(new_time);
+                self.clicked_pos = None;
+            }
+        }
+    }
+    #[allow(unused)]
+    fn draw_old_seeker<B: Backend>(&self, f: &mut Frame<B>, chunk: Rect) {
+        let block = Block::default()
+            .borders(Borders::BOTTOM | Borders::LEFT | Borders::RIGHT)
+            .border_type(BorderType::Rounded);
+
         if self.player.songs.is_empty() {
-            return f.render_widget(
-                Block::default()
-                    .borders(Borders::BOTTOM | Borders::LEFT | Borders::RIGHT)
-                    .border_type(BorderType::Rounded),
-                chunk,
-            );
+            return f.render_widget(block, chunk);
         }
 
-        let area = f.size();
-        let width = area.width;
-        let percent = self.player.seeker();
-        let pos = (f64::from(width) * percent) as usize;
-
-        let mut string: String = (0..width - 6)
+        let width = f.size().width;
+        let ratio = self.player.elapsed() / self.player.duration;
+        let pos = (f64::from(width) * ratio) as usize;
+        let mut string: String = (0..width.saturating_sub(6))
             .map(|i| if (i as usize) < pos { '=' } else { '-' })
             .collect();
 
-        //Place the seeker location
-        if pos < string.len() - 1 {
+        if pos < string.len().saturating_sub(1) {
             string.remove(pos);
             string.insert(pos, '>');
         } else {
@@ -349,11 +361,9 @@ impl Queue {
             string.push('>');
         }
 
-        let p = Paragraph::new(string).alignment(Alignment::Center).block(
-            Block::default()
-                .borders(Borders::BOTTOM | Borders::LEFT | Borders::RIGHT)
-                .border_type(BorderType::Rounded),
-        );
+        let p = Paragraph::new(string)
+            .alignment(Alignment::Center)
+            .block(block);
 
         f.render_widget(p, chunk);
     }
